package com.jeecms.content.convert;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.collect.Lists;
import com.jeecms.common.enums.EnumCommentType;
import com.jeecms.common.enums.EnumContentStatus;
import com.jeecms.common.enums.EnumContentViewCtl;
import com.jeecms.content.dto.*;
import com.jeecms.content.entity.Content;
import com.jeecms.content.entity.ContentAttr;
import com.jeecms.content.entity.ContentAudit;
import com.jeecms.content.entity.ContentDayStatistics;
import com.jeecms.content.entity.ContentExt;
import com.jeecms.content.entity.ContentIndexFail;
import com.jeecms.content.entity.ContentIndexTask;
import com.jeecms.content.entity.ContentMonthStatistics;
import com.jeecms.content.entity.ContentStatistics;
import com.jeecms.content.entity.ContentTxt;
import com.jeecms.content.entity.ContentWeekStatistics;
import com.jeecms.content.dto.ContentTypeSimpleDto;
import com.jeecms.dto.EsContentDto;
import com.jeecms.dto.EsContentQueryDto;
import com.jeecms.dto.EsFrontSearchDto;
import com.jeecms.common.dto.IdNameDto;
import com.jeecms.enums.EnumContentSortType;
import com.jeecms.common.enums.EnumContentTaskType;
import com.jeecms.common.enums.EnumTimeStage;
import com.jeecms.enums.backend.ContentCodeEnum;
import com.jeecms.exception.GlobalException;
import com.jeecms.listener.content.ContentEventEntity;
import com.jeecms.model.instance.ModelInstance;
import com.jeecms.model.instance.ModelInstanceItem;
import com.jeecms.util.JackJsonUtils;
import com.jeecms.common.util.List2MapUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingConstants;
import com.jeecms.common.enums.EnumContentModelBaseItem;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author Admin
 * @Copyright: 杭州藏茗山科技有限公司 All rights reserved.Notice
 * 仅限于授权后使用，禁止非授权传阅以及私自用于商业目的。
 * @datetime 2023/3/9 16:39
 */
@Mapper(componentModel = MappingConstants.ComponentModel.SPRING)
public abstract class ContentConverter {

    public Content convertFromMap(Map<String, Object> source) {
        if (source == null) {
            return null;
        }
        Content c = new Content();
        c.setTitle(getString(source.get(EnumContentModelBaseItem.title.name())));
        if(c.getTitle().length()>200){
            throw GlobalException.of(ContentCodeEnum.CONTENT_TITLE_EXCEED_LENGTH);
        }
        c.setShortTitle(getString(source.get(EnumContentModelBaseItem.shortTitle.name())));
        if(StringUtils.isNotBlank(c.getShortTitle()) && c.getShortTitle().length()>200){
            throw GlobalException.of(ContentCodeEnum.CONTENT_SHORT_TITLE_EXCEED_LENGTH);
        }
        c.setAuthor(getString(source.get(EnumContentModelBaseItem.author.name())));
        if(StringUtils.isNotBlank(c.getAuthor()) && c.getAuthor().length()>200){
            throw GlobalException.of(ContentCodeEnum.CONTENT_AUTHOR_EXCEED_LENGTH);
        }
        c.setIsPublishApp(Boolean.TRUE.equals(
                getBoolean(source.get(EnumContentModelBaseItem.isPublishApp.name()))));
        c.setIsPublishApplet(Boolean.TRUE.equals(
                getBoolean(source.get(EnumContentModelBaseItem.isPublishApplet.name()))));
        c.setIsPublishPc(Boolean.TRUE.equals(
                getBoolean(source.get(EnumContentModelBaseItem.isPublishPc.name()))));
        c.setIsPublishWap(Boolean.TRUE.equals(
                getBoolean(source.get(EnumContentModelBaseItem.isPublishWap.name()))));
        c.setSecretId(getLong(source.get(EnumContentModelBaseItem.secretId.name())));
        c.setPublishTime(getDate(source.get(EnumContentModelBaseItem.publishTime.name())));
        c.setContentSourceName(getString(source.get(EnumContentModelBaseItem.contentSourceName.name())));
        List<Map<String, Object>> headImags = (List<Map<String, Object>>) source.get(
                EnumContentModelBaseItem.headImg.name());
        if(CollectionUtils.isNotEmpty(headImags)) {
            Map<String, Object> headImag = headImags.get(0);
            if (MapUtils.isNotEmpty(headImag)) {
                c.setHeadImag(getString(headImag.get("url")));
                c.setHeadImagId(getLong(headImag.get("id")));
            }
        }else{
            c.setHeadImag("");
            c.setHeadImagId(0L);
        }
        return c;
    }

    private Date getDate(Object o) {
        if (o == null || StringUtils.isBlank(o.toString())) {
            return new Date();
        }
        return DateUtil.parse((String) o);
    }

    public ContentExt convertExtFromMap(Map<String, Object> source) {
        if (source == null) {
            return null;
        }
        ContentExt c = new ContentExt();
        //c.setIndexNum(getString(source.get(EnumContentModelBaseItem.indexNum.name())));
        c.setAreaCode(getString(source.get(EnumContentModelBaseItem.areaCode.name())));
        c.setCityCode(getString(source.get(EnumContentModelBaseItem.cityCode.name())));
        c.setProvinceCode(getString(source.get(EnumContentModelBaseItem.provinceCode.name())));
        c.setSummary(getString(source.get(EnumContentModelBaseItem.summary.name())));
        c.setKeyWord(getString(source.get(EnumContentModelBaseItem.keyWord.name())));
        c.setTplMobile(getString(source.get(EnumContentModelBaseItem.tplMobile.name())));
        c.setTplPc(getString(source.get(EnumContentModelBaseItem.tplPc.name())));
        c.setCommentControl(getInteger(source.get(EnumContentModelBaseItem.commentControl.name())));
        if(c.getCommentControl() == null){
            c.setCommentControl(EnumCommentType.ANYONE_COMMENT.getType());
        }
        c.setViewControl(getInteger(source.get(EnumContentModelBaseItem.viewControl.name())));
        if(c.getViewControl() == null){
            c.setViewControl(EnumContentViewCtl.NO_CTL.getCode());
        }
        Map<String, Object> outLink = (Map<String, Object>) source.get(
                EnumContentModelBaseItem.outLink.name());
        if (MapUtils.isNotEmpty(outLink)) {
            c.setOutLink(getString(source.get("linkUrl")));
            if(StringUtils.isNotBlank(c.getOutLink()) && c.getOutLink().length()>500){
                throw GlobalException.of(ContentCodeEnum.CONTENT_OUTLINK_EXCEED_LENGTH);
            }
            c.setNewWindow(getBoolean(source.get("newWindow")));
        }
        return c;
    }

    protected String getString(Object o) {
        return (String) o;
    }

    public Long getLong(Object o) {
        if (o == null) {
            return null;
        }
        if (o instanceof Long) {
            return (Long) o;
        }
        String str = o.toString();
        if (StringUtils.isNumeric(str)) {
            return Long.valueOf(str);
        }
        return null;
    }

    protected Integer getInteger(Object o) {
        if (o == null) {
            return null;
        }
        if (o instanceof Integer) {
            return (Integer) o;
        }
        String str = o.toString();
        if (StringUtils.isNumeric(str)) {
            return Integer.valueOf(str);
        }
        return null;
    }

    protected Boolean getBoolean(Object o) {
        if (o == null) {
            return null;
        }
        return (Boolean) o;
    }

    @Mapping(target = "id", source = "source1.id")
    @Mapping(target = "createTime", source = "source1.createTime")
    @Mapping(target = "createUser", source = "source1.createUser")
    public abstract ContentEventEntity convert2EventEntity(Content source1, ContentExt source2);

    public abstract ContentPageQueryDto convertPageQuery(ContentQueryDto queryDto,
            Page<Content> page);

    @Mapping(target = "id", source = "source1.id")
    @Mapping(target = "createTime", source = "source1.createTime")
    @Mapping(target = "offlineTime",  expression = "java(getOfflineTime( source1.getStatus(), source1.getUpdateTime()))")
    public abstract ContentPageRecordDto convert2PageRecord(Content source1,
            ContentStatisticsDto source2, String channelName, List<ContentTypeSimpleDto> types,
            Long modelId,String modelName);

    public abstract ContentMultQueryDto convertMultiQueryDto(ContentQueryDto queryDto);
   protected Date getOfflineTime( Integer contentStatus, Date updateTime){
       if(EnumContentStatus.OFFLINE.getStatus().equals(contentStatus)){
           return updateTime;
       }
       return null;
   }
    public ContentMultQueryDto contentMultQuery(ContentQueryDto queryDto) {
        ContentMultQueryDto multQueryDto = convertMultiQueryDto(queryDto);
        if (multQueryDto == null) {
            return multQueryDto;
        }
        EnumContentSortType sortType = EnumContentSortType.getByType(queryDto.getSortType());
        Date now = new Date();
        if (EnumContentSortType.isTotalType(sortType)) {
            multQueryDto.setTableName("jc_content_statistics");
        } else if (EnumContentSortType.isDayType(sortType)) {
            multQueryDto.setTableName("jc_content_day_statistics");
            multQueryDto.setStatisticsTime(ContentConverter.formatDayStatisticsTime(now));
        } else if (EnumContentSortType.isWeekType(sortType)) {
            multQueryDto.setTableName("jc_content_week_statistics");
            multQueryDto.setStatisticsTime(ContentConverter.formatWeekStatisticsTime(now));
        } else if (EnumContentSortType.isMonthType(sortType)) {
            multQueryDto.setTableName("jc_content_month_statistics");
            multQueryDto.setStatisticsTime(ContentConverter.formatMonthStatisticsTime(now));
        } else {
            multQueryDto.setTableName("jc_content_statistics");
        }
        return multQueryDto;
    }

    public static String formatDayStatisticsTime(Date now) {
        return now == null ? null : DateUtil.format(now, DatePattern.NORM_DATE_PATTERN);
    }

    public static String formatWeekStatisticsTime(Date now) {
        return now == null ?
                null :
                DateUtil.format(now, DatePattern.NORM_YEAR_PATTERN) + "(" + DateUtil.weekOfYear(now)
                        + ")";
    }

    public static String formatMonthStatisticsTime(Date now) {
        return now == null ? null : DateUtil.format(now, DatePattern.NORM_MONTH_PATTERN);
    }

    public abstract EsContentQueryDto convertEsPageQuery(ContentQueryDto queryDto);

    public ContentPageRecordDto convert2PageRecord(EsContentDto source, List<IdNameDto> types,
            Integer sortType,String modelName) {
        ContentPageRecordDto pageRecordDto = convert2PageRecordInternal(source, types,modelName);
        if (pageRecordDto == null) {
            return pageRecordDto;
        }
        EnumContentSortType st = EnumContentSortType.getByType(sortType);
        if (EnumContentSortType.isDayType(st)) {
            pageRecordDto.setLikes(source.getDayLikes() == null ? 0 : source.getDayLikes());
            pageRecordDto.setViews(source.getDayViews() == null ? 0 : source.getDayViews());
            pageRecordDto.setComments(
                    source.getDayComments() == null ? 0 : source.getDayComments());
            pageRecordDto.setVisitors(
                    source.getDayVisitors() == null ? 0 : source.getDayVisitors());
        } else if (EnumContentSortType.isWeekType(st)) {
            pageRecordDto.setLikes(source.getWeekLikes() == null ? 0 : source.getWeekLikes());
            pageRecordDto.setViews(source.getWeekViews() == null ? 0 : source.getWeekViews());
            pageRecordDto.setComments(
                    source.getWeekComments() == null ? 0 : source.getWeekComments());
            pageRecordDto.setVisitors(
                    source.getWeekVisitors() == null ? 0 : source.getWeekVisitors());
        } else if (EnumContentSortType.isMonthType(st)) {
            pageRecordDto.setLikes(source.getMonthLikes() == null ? 0 : source.getMonthLikes());
            pageRecordDto.setViews(source.getMonthViews() == null ? 0 : source.getMonthViews());
            pageRecordDto.setComments(
                    source.getMonthComments() == null ? 0 : source.getMonthComments());
            pageRecordDto.setVisitors(
                    source.getMonthVisitors() == null ? 0 : source.getMonthVisitors());
        }
        return pageRecordDto;
    }

    public abstract ContentPageRecordDto convert2PageRecordInternal(EsContentDto source,
            List<IdNameDto> types,String modelName);

    public abstract ContentStatisticsDto convertStatisticsDto(ContentDayStatistics source);

    public abstract ContentStatisticsDto convertStatisticsDto(ContentWeekStatistics source);

    public abstract ContentStatisticsDto convertStatisticsDto(ContentMonthStatistics source);

    public abstract ContentStatisticsDto convertStatisticsDto(ContentStatistics source);

    public abstract Content copy(Content c);

    public abstract ContentDetailDto convert2Detail(Content source);

    public ModelInstance convert2Instance(ContentAttr attr, List<ContentTxt> contentTxts) {
        if (attr == null) {
            return null;
        }
        ModelInstance instance = new ModelInstance();
        instance.setModelId(attr.getModelId());
        List<ModelInstanceItem> items = JackJsonUtils.stringToList(attr.getContent(),
                new TypeReference<List<ModelInstanceItem>>() {
                });
        ContentTxt summary = contentTxts.stream().filter(ContentTxt::getIsSummary).findFirst()
                .orElse(null);
        List<ContentTxt> txts = contentTxts;
        if (summary != null) {
            ModelInstanceItem item = new ModelInstanceItem();
            item.setField(EnumContentModelBaseItem.summary.name());
            item.setValue(summary.getAttrTxt());
            items.add(item);
            txts = contentTxts.stream().filter(txt -> !txt.getIsSummary())
                    .collect(Collectors.toList());
        }
        if (CollectionUtils.isNotEmpty(txts)) {
            ModelInstanceItem item = new ModelInstanceItem();
            item.setField(EnumContentModelBaseItem.contents.name());
            Map<String, String> txtMap = List2MapUtil.list2Map(txts, ContentTxt::getAttrKey,
                    ContentTxt::getAttrTxt, null);
            item.setValue(txtMap);
            items.add(item);
        }
        instance.setInstanceItems(items);
        return instance;
    }

    public IndexNumElement convert2IndexNumElement(Map<String, Object> baseParamMap) {
        if (baseParamMap == null) {
            return null;
        }
        IndexNumElement c = new IndexNumElement();
        Map<String, Object> indexMap = (Map<String, Object>) baseParamMap.get(
                EnumContentModelBaseItem.indexNum.name());
        if (indexMap == null) {
            return null;
        }
        c.setIndexNumRuleIsValid(
                getBoolean(indexMap.get(EnumContentModelBaseItem.indexNumRuleIsValid.name())));
        c.setIndexNumRuleBaseCateCode(
                getLong(indexMap.get(EnumContentModelBaseItem.indexNumRuleBaseCateCode.name())));
        c.setIndexNumRuleIssueCode(
                getLong(indexMap.get(EnumContentModelBaseItem.indexNumRuleIssueCode.name())));
        c.setIndexNumRuleTopicCode(
                getLong(indexMap.get(EnumContentModelBaseItem.indexNumRuleTopicCode.name())));
        return c;
    }

    public EsFrontSearchDto convert2EsQuertDto(FrontSearchDto queryDto) {
        EsFrontSearchDto dto = convert2EsQuertDtoInternal(queryDto);
        if (dto == null) {
            return dto;
        }
        if (StringUtils.isNotBlank(queryDto.getWord())) {
            dto.setKeyWords(
                    Arrays.stream(queryDto.getWord().split(",")).collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(queryDto.getChannelIdStr())) {
            dto.setChannelIds(
                    Arrays.stream(queryDto.getChannelIdStr().split(",")).map(Long::valueOf)
                            .collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(queryDto.getTypeIdStr())) {
            dto.setTypeIds(Arrays.stream(queryDto.getTypeIdStr().split(",")).map(Long::valueOf)
                    .collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(queryDto.getTagIdStr())) {
            dto.setTagIds(Arrays.stream(queryDto.getTagIdStr().split(",")).map(Long::valueOf)
                    .collect(Collectors.toList()));
        }
        Integer timeStage = queryDto.getTimeStage();
        EnumTimeStage stage = EnumTimeStage.getByType(timeStage);
        Date now = new Date();
        switch (stage) {
            case day:
                dto.setPublishTimeStart(DateUtil.beginOfDay(now));
                dto.setPublishTimeEnd(now);
                break;
            case week:
                dto.setPublishTimeStart(DateUtil.beginOfWeek(now));
                dto.setPublishTimeEnd(now);
                break;
            case month:
                dto.setPublishTimeStart(DateUtil.beginOfMonth(now));
                dto.setPublishTimeEnd(now);
                break;
            case year:
                dto.setPublishTimeStart(DateUtil.beginOfYear(now));
                dto.setPublishTimeEnd(now);
                break;
            default:
        }
        return dto;
    }

    public EsFrontSearchDto convert2EsQuertDto(FrontPageQueryDto queryDto) {
        EsFrontSearchDto dto = convert2EsQuertDtoInternal(queryDto);
        if (dto == null) {
            return dto;
        }
        if(queryDto.getSiteId()!=null){
            dto.setSiteIds(Lists.newArrayList(queryDto.getSiteId()));
        }
        if (StringUtils.isNotBlank(queryDto.getChannelIdStr())) {
            dto.setChannelIds(
                    Arrays.stream(queryDto.getChannelIdStr().split(",")).map(Long::valueOf)
                            .collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(queryDto.getTypeIdStr())) {
            dto.setTypeIds(Arrays.stream(queryDto.getTypeIdStr().split(",")).map(Long::valueOf)
                    .collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(queryDto.getExcludeTypeIdStr())) {
            dto.setExcludeTypeIds(Arrays.stream(queryDto.getExcludeTypeIdStr().split(",")).map(Long::valueOf)
                    .collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(queryDto.getTagIdStr())) {
            dto.setTagIds(Arrays.stream(queryDto.getTagIdStr().split(",")).map(Long::valueOf)
                    .collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(queryDto.getModelId())) {
            dto.setModelIds(Arrays.stream(queryDto.getModelId().split(",")).map(Long::valueOf)
                    .collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(queryDto.getExcludeId())) {
            dto.setExcludeIds(Arrays.stream(queryDto.getExcludeId().split(",")).map(Long::valueOf)
                    .collect(Collectors.toList()));
        }
        if (Objects.equals(1, queryDto.getReleaseTarget())) {
            dto.setReleasePc(true);
        }
        if (Objects.equals(2, queryDto.getReleaseTarget())) {
            dto.setReleaseWap(true);
        }
        if (queryDto.getTimeBegin() != null) {
            dto.setPublishTimeStart(queryDto.getTimeBegin());
        }
        if (queryDto.getTimeEnd() != null) {
            dto.setPublishTimeEnd(queryDto.getTimeEnd());
        }
        return dto;
    }

    protected abstract EsFrontSearchDto convert2EsQuertDtoInternal(FrontPageQueryDto queryDto);
    @Mapping(source = "wordMatchType", target = "keyWordMatchType")
    @Mapping(source = "wordField", target = "keyWordField")
    public abstract EsFrontSearchDto convert2EsQuertDtoInternal(FrontSearchDto queryDto);

    public abstract ContentIdTitleDto content2IdTitleDto(Content content);

    @Mapping(source = "source2", target = "tasks")
    public abstract ContentIdTitleDto content2IdTitleDtoInternal(Content source1,
            List<TrContentTaskDto> source2);

    public ContentIdTitleDto content2IdTitleDto(Content source1, List<TrContentTaskDto> source2) {
        ContentIdTitleDto titleDto = content2IdTitleDtoInternal(source1, source2);
        if (titleDto == null) {
            return titleDto;
        }
        List<TrContentTaskDto> tasks = titleDto.getTasks();
        if (CollectionUtils.isEmpty(tasks)) {
            titleDto.setStatuses(Lists.newArrayList(source1.getStatus()));
        } else {
            List<Integer> status = tasks.stream().map(item -> {
                Integer taskType = item.getTaskType();
                if (EnumContentTaskType.TOP_TASK.getType().equals(taskType)) {
                    return 6;
                } else if (EnumContentTaskType.PUBLISH_TASK.getType().equals(taskType)) {
                    return 7;
                }
                return 8;
            }).collect(Collectors.toList());
            status.add(source1.getStatus());
            titleDto.setStatuses(status);
        }
        return titleDto;
    }

    public ContentlistPageQueryDto convertPageQueryDto(ContentlistQueryDto queryDto) {
        if (queryDto == null) {
            return null;
        }

        ContentlistPageQueryDto contentlistPageQueryDto = new ContentlistPageQueryDto();
        contentlistPageQueryDto.setTitle(queryDto.getTitle());
        contentlistPageQueryDto.setChannelId(queryDto.getChannelId());
        if (queryDto.getCreateTimeStart() != null || queryDto.getCreateTimeEnd() != null) {
            contentlistPageQueryDto.setCreateTime(
                    Lists.newArrayList(queryDto.getCreateTimeStart(), queryDto.getCreateTimeEnd()));
        }
        if (queryDto.getPublishTimeStart() != null || queryDto.getPublishTimeEnd() != null) {
            contentlistPageQueryDto.setPublishTime(
                    Lists.newArrayList(queryDto.getPublishTimeStart(),
                            queryDto.getPublishTimeEnd()));
        }
        return contentlistPageQueryDto;
    }

    public abstract ContentSortPageRecord content2SortPageRecordDto(Content content,
            String channelName);

    public abstract ContentAudit convert2Audit(ContentAuditEn auditDto);

    public abstract ContentIndexTaskDto content2IndexTaskDto(ContentIndexTask task);

    public abstract ContentIndexFailDto convert2FailDto(ContentIndexFail record, String channelName);
}
